(* ---------------------------------------------- *)
(* EW-CheckBox v. 1.0                             *)
(* ---------------------------------------------- *)
(* Author : Steffen "MasterEvil" Goldenstein      *)
(* Mail   : webmaster@evils-world.de              *)
(* Web    : www.evils-world.de                    *)
(* ---------------------------------------------- *)
(* (c) EW-Projects                                *)
(* ---------------------------------------------- *)

unit EwCheckBox;

{$IFDEF VER90}
  {$DEFINE BD2}
{$ELSE}
  {$IFDEF VER100}
    {$DEFINE BD3}
  {$ELSE}
    {$IFDEF VER120}
      {$DEFINE BD4}
    {$ELSE}
      {$DEFINE BD5}
    {$ENDIF}
  {$ENDIF}
{$ENDIF}

{$IFDEF VER140}
  {$DEFINE BPDJ_D6}
{$ENDIF}

interface

{$R EwCheckBox.res}

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  //Dialogs;
  ExtCtrls;
  //MMSystem;

type
  TOnMouseOverEvent = procedure(Sender: TObject) of object;
  TOnMouseOutEvent = procedure(Sender: TObject) of object;
  TLeft = (taRightJustify,taLeftJustify);
  TMargine = (bmNone,bmLowered,bmRaised);
  TImageType = (igCheck,igCross,igRound);

  TEwCustomCheck = class(TCustomControl)
  private
    VHotColor     : TColor;
    R             : TRect;
    CaptRect      : TRect;
{$IFDEF BD3}
{$ELSE}
    BiDiFlags     : Longint;
{$ENDIF}
    VShowFocused  : Boolean;
    VColor        : TColor;
    VFlat         : Boolean;
    VRound        : Boolean;
    VTransparent  : Boolean;
    VTransColor   : TColor;
    Index         : Integer;
    VChecked      : Boolean;
    VHot          : Boolean;
    VGlyph        : TImageType;
    VMargine      : TMargine;
    VLeft         : TLeft;
    VBitmap       : TBitmap;
    VOnMouseOver  : TOnMouseOverEvent;
    VOnMouseOut   : TOnMouseOutEvent;

    procedure DrawParentImage(Control: TControl; Dest: TCanvas);
    procedure SetTransparent(Value: Boolean);
    procedure SetShowFocused(Value: Boolean);
    procedure SetColor(Value: TColor);
    procedure SetFlat(Value: Boolean);
    procedure SetRound(Value: Boolean);
    procedure SetHot(Value: Boolean);
    procedure SetHotColor(Value: TColor);
    procedure SetMargine(Value: TMargine);
    procedure DrawCaptionEnabled;
    procedure DrawMargin;
    procedure DrawImage;
    procedure DrawImageHot;
    procedure DrawCaption;
    procedure SetGlyph(Value: TImageType);
    procedure DoClick(Sender: TObject);
    procedure CMMouseEnter(var AMsg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var AMsg: TMessage); message CM_MOUSELEAVE;
    procedure SetChecked(Value: Boolean);
    procedure SetLeft(Value: TLeft);
    procedure CmEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure CmParentColorChanged(var Message: TWMNoParams); message CM_PARENTCOLORCHANGED;
    procedure CmTextChanged(var Message: TWmNoParams); message CM_TEXTCHANGED;
    procedure CmVisibleChanged(var Message: TWmNoParams); message CM_VISIBLECHANGED;
    procedure CmParentFontChanged(var Message: TWMNoParams); message CM_FONTCHANGED;
    procedure WMLButtonDblClk (var Message: TWMLButtonDown); message WM_LBUTTONDBLCLK;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure CmDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
  protected
    procedure Click; override;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    property ShowFocused: Boolean read VShowFocused write SetShowFocused default true;
    property Color: TColor read VColor write SetColor default clBtnFace;
    property Flat: Boolean read VFlat write SetFlat default false;
    property Round: Boolean read VRound write SetRound default false;
    property Hot: Boolean read VHot write SetHot default false;
    property HotColor: TColor read VHotColor write SetHotColor default clWindowText;
    property OnMouseEnter: TOnMouseOverEvent read VOnMouseOver write VOnMouseOver;
    property OnMouseExit: TOnMouseOutEvent read VOnMouseOut write VOnMouseOut;
    property Checked: Boolean read VChecked write SetChecked default false;
    property Alignment: TLeft read VLeft write SetLeft default taRightJustify;
    property Glyph: TImageType read VGlyph write SetGlyph default igCheck;
    property Style: TMargine read VMargine write SetMargine default bmNone;
    property Transparent: Boolean read VTransparent write SetTransparent default false;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TEwCheckBox = class(TEwCustomCheck)
  published
     property Transparent;
     property ShowFocused;
     property Flat;
     property Round;
     property Hot;
     property HotColor;
     property OnMouseEnter;
     property OnMouseExit;
     property Checked;
     property Alignment;
     property Glyph;
     property Style;
     property Enabled;
     property DragCursor;
     property DragMode;
     property Caption;
     property Font;
     property ParentFont;
     property Color;
     property ParentColor;
     property Visible;
     property ShowHint;
     property OnMouseDown;
     property OnMouseUp;
     property OnMouseMove;
     property OnClick;
     property OnDragDrop;
     property OnDragOver;
     property OnEndDrag;
     property OnStartDrag;
     property OnEnter;
     property OnExit;
     property TabStop default true;
     property TabOrder;
     property OnKeyDown;
     property OnKeyPress;
     property OnKeyUp;
     property PopupMenu;
{$IFDEF BD5}
     property OnContextPopup;
{$ENDIF}
{$IFDEF DD3}
{$ELSE}
     property DragKind;
     property Anchors;
     property Constraints;
     property BiDiMode;
     property ParentBiDiMode;
     property OnStartDock;
     property OnEndDock;
{$ENDIF}
  end;
  
procedure Register;

implementation

var
  Pattern         : TBitmap = nil;

procedure Register;
begin
  RegisterComponents('Effets', [TEwCheckBox]);
end;

procedure TEwCustomCheck.SetShowFocused(Value: Boolean);
begin
  if Value<>VShowFocused then VShowFocused := Value;
end;

procedure TEwCustomCheck.SetTransparent(Value: Boolean);
begin
  if Value<>VTransparent then
  begin
    VTransparent := Value;
    Repaint;
  end;
end;


procedure TEwCustomCheck.DoClick(Sender: TObject);
begin
 if Assigned(OnClick) then OnClick(Self);
end;

procedure TEwCustomCheck.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
  begin
    if (IsAccel(CharCode,Caption)) and (CanFocus) then
    begin
      Click;
      Result := 1;
      SetFocus;
      if VChecked then SetChecked(false)
      else SetChecked(true);
    end else inherited;
  end;
end;

procedure TEwCustomCheck.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  Repaint;
end;

procedure TEwCustomCheck.WMKillFocus(var Message: TWMKillFocus);
begin
  inherited;
  Repaint;
end;

procedure TEwCustomCheck.DoEnter;
begin
  inherited DoEnter;
end;

procedure TEwCustomCheck.DoExit;
begin
  inherited DoExit;
end;

procedure TEwCustomCheck.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Key = VK_RETURN then DoClick(Self);
  inherited KeyDown(Key,Shift);
end;

procedure TEwCustomCheck.SetColor(Value: TColor);
begin
  if Value<>VColor then
  begin
    VColor := Value;
    Repaint;
  end;
end;

procedure TEwCustomCheck.SetFlat(Value: Boolean);
begin
  if Value<>VFlat then
  begin
    VFlat := Value;
    Repaint;
  end;
end;

procedure TEwCustomCheck.SetRound(Value: Boolean);
begin
  if Value<>VRound then
  begin
    VRound := Value;
    Repaint;
  end;
end;

procedure TEwCustomCheck.SetHot(Value: Boolean);
begin
  if Value<>VHot then
  begin
    VHot := Value;
    Repaint;
  end;
end;

procedure TEwCustomCheck.SetHotColor(Value: TColor);
begin
  if Value<>VHotColor then
  begin
    VHotColor := Value;
    Repaint;
  end;
end;

procedure TEwCustomCheck.SetLeft(Value: TLeft);
begin
  if Value<>VLeft then
  begin
    VLeft := Value;
    RecreateWnd;
  end;
end;

procedure TEwCustomCheck.Click;
begin
  inherited;
end;

procedure TEwCustomCheck.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button<>mbleft then exit
  else
  begin
    if not Enabled then exit;
    SetFocus;
    if not VChecked then
    begin
      VChecked := true;
      Repaint;
    end else
    begin
      VChecked:=false;
      Repaint;
    end;
  end;
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TEwCustomCheck.WMLButtonDblClk(var Message: TWMLButtonDown);
begin
  inherited;
  Click;
end;

procedure TEwCustomCheck.SetChecked(Value: Boolean);
begin
  VChecked := Value;
  Invalidate;
end;

procedure TEwCustomCheck.SetMargine(Value: TMargine);
begin
  if Value<>VMargine then
  begin
    VMargine := Value;
    Invalidate;
  end;
end;

procedure TEwCustomCheck.DrawParentImage(Control: TControl; Dest: TCanvas);
var
  SaveIndex       : Integer;
  DC              : HDC;
  Position        : TPoint;
begin
  with Control do
  begin
    if Parent=nil then Exit;
    DC := Dest.Handle;
    SaveIndex := SaveDC(DC);
{$IFDEF BD2}
    GetViewportOrgEx(DC,@Position);
{$ELSE}
    GetViewportOrgEx(DC,Position);
{$ENDIF}
    SetViewportOrgEx(DC,Position.X-Left,Position.Y-Top,nil);
    IntersectClipRect(DC,0,0,Parent.ClientWidth,Parent.ClientHeight);
    Parent.Perform(WM_ERASEBKGND,Integer(DC),Integer(0));
    Parent.Perform(WM_PAINT,Integer(DC),Integer(0));
    RestoreDC(DC,SaveIndex);
  end;
end;

procedure TEwCustomCheck.Paint;
var
  FocusRect       : TRect;
  z               : Integer;
  n               : Integer;
begin
  DrawMargin;
  if VTransparent then DrawParentImage(Self,Self.Canvas);
  with Canvas do
  begin
    z := (R.Top+((R.Bottom-R.Top)-TextHeight(Caption)) div 2);
    if Pos('&',Caption)>0 then n := TextWidth('&')
    else n := 0;
    case VLeft of
      taRightJustify: FocusRect := Rect(R.Left+22,z,R.Left+22+TextWidth(Caption)-n+3,z+TextHeight(Caption));
      taLeftJustify : FocusRect := Rect(R.Left+2,z,R.Left+2+TextWidth(Caption)-n+3,z+TextHeight(Caption));
    end;
  end;
  if ShowFocused then
  begin
    if Focused then DrawFocusRect(Canvas.Handle,Rect(FocusRect.Left-1,FocusRect.Top-1,FocusRect.Right+1,FocusRect.Bottom+1));
  end;
  DrawCaption;
  DrawImage;
  if VChecked then DrawImageHot;
end;

procedure TEwCustomCheck.DrawMargin;
begin
  R := ClientRect;
  Canvas.Brush.Color := VColor;
  Canvas.Brush.Style := bsSolid;
  case VMargine of
    bmNone:
    begin
      Frame3D(Canvas,R,clBtnFace,clBtnFace,0);
    end;
    bmLowered:
    begin
      Frame3D(Canvas,R,clBtnShadow,clBtnHighlight,1);
    end;
    bmRaised:
    begin
      Frame3D(Canvas,R,clBtnHighlight,clBtnShadow,1);
    end;
  end;
  Canvas.FillRect(R);
end;

procedure TEwCustomCheck.CMMouseEnter(var AMsg: TMessage);
begin
  if not Enabled then Exit;
  if VHot then
  begin
    Index := 1;
    Repaint;
  end;
  if Assigned(VOnMouseOver) then VOnMouseOver(Self);
end;

procedure TEwCustomCheck.CMMouseLeave(var AMsg: TMessage);
begin
  if not Enabled then Exit;
  if VHot then
  begin
    Index:=0;
    Repaint;
  end;
  if Assigned(VOnMouseOut) then VOnMouseOut(Self);
end;

procedure TEwCustomCheck.DrawCaption;
var
  X               : Integer;
begin
  with Canvas do
  begin
    Font := Self.Font;

    if VHot then
    begin
      if Index=1 then Font.Color := VHotColor
    end;

    Brush.Style := bsClear;
    x := (R.Top+((R.Bottom-R.Top)-TextHeight(Caption)) div 2);
    case VLeft of
      taRightJustify: CaptRect := Rect(R.Left+22,x,R.Left+22+TextWidth(Caption),x+TextHeight(Caption));
      taLeftJustify : CaptRect := Rect(R.Left+2,x,R.Left+2+TextWidth(Caption),x+TextHeight(Caption));
    end;
    if not Enabled then DrawCaptionEnabled;
{$IFDEF BD2}
    DrawText(Handle,PChar(Caption),Length(Caption),CaptRect,DT_LEFT);
{$ENDIF}
{$IFDEF BD3}
    DrawText(Handle,PChar(Caption),Length(Caption),CaptRect,DT_LEFT);
{$ELSE}
    DrawText(Handle,PChar(Caption),Length(Caption),CaptRect,DT_LEFT or BiDiFlags);
{$ENDIF}
  end;
end;

procedure TEwCustomCheck.DrawCaptionEnabled;
var
  ECaptRect       : TRect;
begin
  with Canvas do
  begin
    Font := Self.Font;
    Brush.Style := bsClear;
{$IFDEF BD2}
    DrawText(Handle,PChar(Caption),Length(Caption),CaptRect,DT_LEFT);
{$ENDIF}
{$IFDEF BD3}
    DrawText(Handle,PChar(Caption),Length(Caption),CaptRect,DT_LEFT);
{$ELSE}
    DrawText(Handle,PChar(Caption),Length(Caption),CaptRect,DT_LEFT or BiDiFlags);
{$ENDIF}
    Font.Color := VHotColor;

    ECaptRect := Rect(CaptRect.Left+1,CaptRect.Top+1,CaptRect.Right+1,CaptRect.Bottom+1);
{$IFDEF BD2}
    DrawText(Handle,PChar(Caption),Length(Caption),ECaptRect,DT_LEFT);
{$ENDIF}
{$IFDEF BD3}
    DrawText(Handle,PChar(Caption),Length(Caption),ECaptRect,DT_LEFT);
{$ELSE}
    DrawText(Handle,PChar(Caption),Length(Caption),ECaptRect,DT_LEFT or BiDiFlags);
{$ENDIF}
    Font.Color := clBtnShadow;
  end;
end;

procedure TEwCustomCheck.DrawImage;
var
  BitRect         : TRect;
  Image           : TRect;
  x               : Integer;
begin
  if VFlat then
  begin
    if VRound then
    begin
      if Enabled then VBitmap.Handle := LoadBitmap(hInstance, 'BlankFlatRound')
      else VBitmap.Handle := LoadBitmap(hInstance, 'DisabledFlatRound');
    end else
    begin
      if Enabled then VBitmap.Handle := LoadBitmap(hInstance, 'BlankFlat')
      else VBitmap.Handle := LoadBitmap(hInstance, 'DisabledFlat');
    end;
  end else
  begin
    if VRound then
    begin
      if Enabled then VBitmap.Handle := LoadBitmap(hInstance, 'BlankRound')
      else VBitmap.Handle := LoadBitmap(hInstance, 'DisabledRound');
    end else
    begin
      if Enabled then VBitmap.Handle := LoadBitmap(hInstance, 'Blank')
      else VBitmap.Handle := LoadBitmap(hInstance, 'Disabled');
    end;
  end;
  VTransColor := VBitmap.Canvas.Pixels[0,VBitmap.Height-1];
  Image := Rect(0,0,VBitmap.Width,VBitmap.Height);
  x := (R.Top+((R.Bottom-R.Top)-VBitmap.Height) div 2);
  case VLeft of
    taRightJustify: BitRect := Rect(R.Left+2,x,R.Left+2+VBitmap.Width,x+VBitmap.Height);
    taLeftJustify : BitRect := Rect(R.Right-18,x,R.Right-18+VBitmap.Width,x+VBitmap.Height);
  end;
{$IFDEF BD2}
  Canvas.BrushCopy(Bitrect,VBitmap,Image,clFuchsia);
{$ELSE}
  Canvas.BrushCopy(Bitrect,VBitmap,Image,VTransColor);
{$ENDIF}
end;

procedure TEwCustomCheck.SetGlyph(Value: TImageType);
begin
  if Value<>VGlyph then
  begin
    VGlyph := Value;
    RecreateWnd;
  end;
end;

procedure TEwCustomCheck.DrawImageHot;
var
  BitHotRect      : TRect;
  ImageHot        : TRect;
  x               : Integer;
begin
  case VGlyph of
    igCheck:
    begin
      if Enabled then VBitmap.Handle := LoadBitmap(hInstance,'ImCheck')
      else
      begin
        if (VFlat) and (not VRound) then VBitmap.Handle := LoadBitmap(hInstance,'DisabledFlat')
        else if (VRound) and (not VFlat) then VBitmap.Handle := LoadBitmap(hInstance,'DisabledRound')
        else VBitmap.Handle := LoadBitmap(hInstance,'Disabled');
      end;
    end;
    igCross:
    begin
      if Enabled then VBitmap.Handle := LoadBitmap(hInstance,'ImCross')
      else
      begin
        if (VFlat) and (not VRound) then VBitmap.Handle := LoadBitmap(hInstance,'DisabledFlat')
        else if (VRound) and (not VFlat) then VBitmap.Handle := LoadBitmap(hInstance,'DisabledRound')
        else VBitmap.Handle := LoadBitmap(hInstance,'Disabled');
      end;
    end;
    igRound:
    begin
      if Enabled then VBitmap.Handle := LoadBitmap(hInstance,'ImRound')
      else
      begin
        if (VFlat) and (not VRound) then VBitmap.Handle := LoadBitmap(hInstance,'DisabledFlat')
        else if (VRound) and (not VFlat) then VBitmap.Handle := LoadBitmap(hInstance,'DisabledRound')
        else VBitmap.Handle := LoadBitmap(hInstance,'Disabled');
      end;
    end;
  end;
  VTransColor := VBitmap.Canvas.Pixels[0,VBitmap.Height-1];
  ImageHot := Rect(0,0,VBitmap.Width,VBitmap.Height);
  x := (R.Top+((R.Bottom-R.Top)-VBitmap.Height) div 2);
  case VLeft of
    taRightJustify: BitHotRect := Rect(R.Left+2,x,R.Left+2+VBitmap.Width,x+VBitmap.Height);
    taLeftJustify : BitHotRect := Rect(R.Right-18,x,R.Right-18+VBitmap.Width,x+VBitmap.Height);
  end;
{$IFDEF BD2}
  Canvas.BrushCopy(BitHotrect,VBitmap,ImageHot,clFuchsia);
{$ELSE}
  canvas.BrushCopy(BitHotrect,VBitmap,ImageHot,VTransColor);
{$ENDIF}
end;

procedure TEwCustomCheck.CmParentFontChanged(var Message: TWMNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TEwCustomCheck.CmTextChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TEwCustomCheck.CmVisibleChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TEwCustomCheck.CmParentColorChanged(var Message: TWMNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TEwCustomCheck.CmEnabledChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

constructor TEwCustomCheck.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  SetBounds(0, 0, 97, 19);
  VChecked := false;
  VColor := clBtnFace;
  VLeft := taRightJustify;
  VGlyph := igCheck;
  VMargine := bmNone;
  VHot := false;
  VHotColor := clHighlight;
  Index := 0;
  VFlat := false;
  VRound := false;
  VShowFocused := true;
  TabStop := true;
  VBitmap := TBitmap.Create;
end;

destructor TEwCustomCheck.Destroy;
begin
  inherited Destroy;
  VBitmap.Free;
end;

end.
